# Microeconometrics
# CLASS 9e
# Simulations
# Wiktor Budzinski

library(pscl)
library(tseries)
library(nlWaldTest)
library(ramify)
library(tidyr)
library(mvtnorm)
library(plm)

############################ Exercise 2 ##################################
# Simulation for the Voung test
# "Baby" Monte Carlo
rm(list=ls())
set.seed(1234567) # setting seed
N<-100

x1 <- runif(N,0,1)
y<-rpois(N, exp(0.5+1*x1))
m1 <- glm(y~x1,  family = poisson)
m2 <- zeroinfl(y~x1, dist="poisson")

lp<-y*log(m1$fitted.values)-m1$fitted.values-lgamma(y+1)
lam2 <- predict(m2, type = "count")
p0 <- predict(m2, type = "zero")
p2_tmp<- y*log(lam2)-lam2-lgamma(y+1)
p2 <- (1-p0)*exp(p2_tmp)
p2[y==0] <- p2[y==0] + p0[y==0]
lp2<- log(p2)

Voung <- sqrt(N)*mean(lp2-lp)/sd(lp2-lp)
Voung

## Actual Monte Carlo simulation

set.seed(1234567) # setting seed
iter <- 200                       # number of iterations
N<-200
Vmat <- matrix(, nrow = iter)
Conv <-matrix(, nrow = iter)
  
for (i in 1:iter) {
  
  x1 <- runif(N,0,1)
  y<-rpois(N, exp(0.5+1*x1))
  m1 <- glm(y~x1,  family = poisson)
  m2 <- zeroinfl(y~x1 | 1, dist="poisson", method = ("Nelder-Mead"))
  Conv[i]<-m2$converged
  
  # Poisson LL
  lp<-y*log(m1$fitted.values)-m1$fitted.values-lgamma(y+1) 
  # ZIP LL
  lam2 <- predict(m2, type = "count")
  p0 <- predict(m2, type = "zero")
  p2_tmp<- y*log(lam2)-lam2-lgamma(y+1)
  p2 <- (1-p0)*exp(p2_tmp)
  p2[y==0] <- p2[y==0] + p0[y==0]
  lp2<- log(p2)
  
  Vmat[i]<- sqrt(N)*mean(lp2-lp)/sd(lp2-lp)
}

mean(Conv)
hist(Vmat[Conv])
jarque.bera.test(Vmat[Conv])

## Actually non-nested models
set.seed(1234567) # setting seed
iter <- 200                        # number of iterations
Vmat <- matrix(, nrow = iter)
N<-200
start_time = Sys.time()

for (i in 1:iter) {
  
  x1 <- runif(N,-2,2)
  x2 <- runif(N,-2,2)
  y<-rpois(N, exp(0.5+1*x1 + x2))
  m1 <- glm(y~x1,  family = poisson)
  m2 <- glm(y~x2,  family = poisson)
  
  lp<-y*log(m1$fitted.values)-m1$fitted.values-lgamma(y+1)
  lp2<-y*log(m2$fitted.values)-m2$fitted.values-lgamma(y+1)
  
  Vmat[i]<- sqrt(N)*mean(lp2-lp)/sd(lp2-lp)
}
hist(Vmat)
jarque.bera.test(Vmat)

############################ Exercise 3 ##################################
# Comparing the efficiency of One-Step and Two-step GMM

rm(list=ls())
GenPanel<-function(N,TT, rho){

  id<- c(t(kronecker(matrix(1,1,TT),1:N)))
  
  # Independent variables
  fe <- rnorm(N,0,1) # fixed effect
  fe2<-c(t(kronecker(matrix(1,1,TT+1),fe)))
  x1 <- (-1*fe2 + rnorm((TT+1)*N,0,1))/2
  x2 <- rnorm((TT+1)*N,0,1)
  
  Fit <- x1*0.5 -x2 + fe2 # XB
  Fit <-matrix(Fit, nrow = (TT+1), ncol = N)
  A <-diag(TT+1) 
  A[2:(TT+1),1:TT] <- A[2:(TT+1),1:TT]-rho*diag(TT)
  Mat <- inv(A)
  SIG<- diag(TT+1)
  VAR<- Mat %*% SIG %*% t(Mat) # Variance
  
  Means <- Mat %*% Fit # Means
  
  y <- matrix(, nrow = N, ncol = TT+1) # Dependent variable
  for(i in 1:N){
    y[i,] <- rmvnorm(n=1, mean=t(Means[,i]), sigma=VAR)
  }
  # Some data transformation
  y <- matrix(t(y[,2:(TT+1)]), nrow = TT*N, ncol = 1)
  x1 <- matrix(x1, nrow = (TT+1), ncol = N)
  x1 <- matrix(x1[2:(TT+1),], nrow = TT*N, ncol = 1)
  x2 <- matrix(x2, nrow = (TT+1), ncol = N)
  x2 <- matrix(x2[2:(TT+1),], nrow = TT*N, ncol = 1)
  
  data<-data.frame(id, y,x1, x2)
  return(data)
}

set.seed(1234567) # setting seed
iter<-500
gmm_mat <- matrix(, nrow = iter, ncol = 6)
for (i in 1:iter) {
  D<-GenPanel(300, 6, 0.6)
  Panel_data<-pdata.frame(D,index = c("id"))
  #gmm1 <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:6), data = Panel_data, model = "onestep")
  gmm2 <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:6), data = Panel_data, model = "twosteps")
  
 # gmm_mat[i,1:3]<-gmm1$coefficients[1:3]
  gmm_mat[i,1:3]<-gmm2$coefficients[[1]][1:3]
  gmm_mat[i,4:6]<-gmm2$coefficients[[2]][1:3]
}
apply(gmm_mat,2,sd)
# Are they closer to true values
apply(gmm_mat,2,mean)

### What if residuals are AR(1) ?
GenPanelAR<-function(N,TT, rho, rho2){
  
  id<- c(t(kronecker(matrix(1,1,TT),1:N)))
  # Independent variables
  fe <- rnorm(N,0,1) # fixed effect
  fe2<-c(t(kronecker(matrix(1,1,TT+1),fe)))
  x1 <- (-1*fe2 + rnorm((TT+1)*N,0,1))/2
  x2 <- rnorm((TT+1)*N,0,1)
  
  Fit <- x1*0.5 -x2 + fe2 # XB
  Fit <-matrix(Fit, nrow = (TT+1), ncol = N)
  
  A <-diag(TT+1) 
  A[2:(TT+1),1:TT] <- A[2:(TT+1),1:TT]-rho*diag(TT)
  Mat <- inv(A)
  SIG<- diag(TT+1)
  SIG[2:(TT+1),1:TT] <- SIG[2:(TT+1),1:TT]+rho2*diag(TT)
  SIG[1:TT, 2:(TT+1)] <- SIG[1:TT, 2:(TT+1)]+rho2*diag(TT)
  
  VAR<- Mat %*% SIG %*% t(Mat) # Variance
  Means <- Mat %*% Fit # Means
  
  y <- matrix(, nrow = N, ncol = TT+1) # Dependent variable
  for(i in 1:N){
    y[i,] <- rmvnorm(n=1, mean=t(Means[,i]), sigma=VAR)
  }
  # Some data transformation
  y <- matrix(t(y[,2:(TT+1)]), nrow = TT*N, ncol = 1)
  x1 <- matrix(x1, nrow = (TT+1), ncol = N)
  x1 <- matrix(x1[2:(TT+1),], nrow = TT*N, ncol = 1)
  x2 <- matrix(x2, nrow = (TT+1), ncol = N)
  x2 <- matrix(x2[2:(TT+1),], nrow = TT*N, ncol = 1)
  
  data<-data.frame(id, y,x1, x2)
  return(data)
}

set.seed(1234567) # setting seed
iter<-500
gmm_mat2 <- matrix(, nrow = iter, ncol = 6)
for (i in 1:iter) {
  D<-GenPanelAR(300, 6, 0.6, 0.4)
  Panel_data<-pdata.frame(D,index = c("id"))
  #gmm1 <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:6), data = Panel_data, model = "onestep")
  gmm2 <- pgmm(y ~ lag(y, 1) + x1 + x2 | lag(y, 2:6), data = Panel_data, model = "twosteps")
  
  # gmm_mat[i,1:3]<-gmm1$coefficients[1:3]
  gmm_mat2[i,1:3]<-gmm2$coefficients[[1]][1:3]
  gmm_mat2[i,4:6]<-gmm2$coefficients[[2]][1:3]
}
apply(gmm_mat2,2,sd)
apply(gmm_mat2,2,mean)

hist(gmm_mat2[,1])

############################ Exercise 4 ##################################
# Using simulations to test for zero-inflation

library(MASS)

rm(list=ls())
setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2024\\Class 9B")         # sets the access path (sometimes change the characters '\' to '/' is needed)
baltic  <-  readRDS("baltic.rds")

Eq1 <- trips ~ dk + ee + fi + lit + lv + pl + ruc + se + age + bocc+ male + factor(edu) + hhsize+  tc
model1 <- glm(Eq1, data=baltic, family = poisson)
summary(model1)

model2 <- glm.nb(Eq1, data=baltic)
summary(model2)
LR <- 2 * (logLik(model2) - logLik(model1))
pchisq(LR, df = 1, lower.tail = FALSE)

hist(baltic$trips)
mean(baltic$trips == 0)

# How to obtain critical value for this statstic? (Share of 0s)
#x1<-runif(1000, 0,1)
  ##y<-rnbinom(1000, exp(-0.5 + x1), 0.9)
#y<-rnbinom(1000, mu = exp(-0.5 + x1), size = 1.5)
#modelX <- glm.nb(y~x1)
#summary(modelX)

# H0: No zero inflation

Iter <- 5000
set.seed(1234567) # setting seed
length(model2$fitted.values)
Mat_Zeros <- matrix(, nrow = Iter, ncol = 1)
for (i in 1:Iter) {
  y_i <- rnbinom(8655, model2$fitted.values, model2$theta)
  Mat_Zeros[i] <- mean(y_i == 0)
}
hist(Mat_Zeros)

quantile(Mat_Zeros, probs = 0.99)
mean(baltic$trips == 0)